unit fastgraph;

{

http://www.freepascal.org/docs-html/rtl/graph/index-5.html

Circle
Ellipse
GetPixel
InitGraph
LineTo
Line
Locate
MoveTo
PutPixel
Rectangle
SetBkColor
SetColor

}

interface


implementation

var	GraphResult: byte;

	color_bits: array [0..$3ff] of byte;
	lineLo, lineHi, div4: array [0..255] of byte;

	tmpX0, tmpY0: smallint;

const
	D2bit = 12;
	D4bit = 13;
	D6bit = 14;
	D8bit = 15;

	m640x480 = 8;
	
	grOK = 0;


procedure SetBkColor(color: byte); assembler;
//----------------------------------------------------------------------------------------------
// Sets the background color to Color
//----------------------------------------------------------------------------------------------
asm
{	mva color colbaks
};
end;


procedure SetColor(color: byte); assembler;
//----------------------------------------------------------------------------------------------
// Sets the foreground color to Color
//----------------------------------------------------------------------------------------------
asm
{	jmp gr8
mode	equ *-2

gr15	lda color
	and #3
	tay

	jmp toend

colorHi		dta h(adr.color_bits, adr.color_bits+$100, adr.color_bits+$200, adr.color_bits+$300)

gr8	lda color
	and #1
	tay

toend
	.ifdef fLine
	lda colorHi,y
	sta fLine.urr_color+2
	sta fLine.uur_color+2
	sta fLine.drr_color+2
	sta fLine.ddr_color+2

	lda #$1d		; ora *,x
	cpy #0
	sne
	lda #$3d		; and *,x

	sta fLine.urr_color
	sta fLine.uur_color
	sta fLine.drr_color
	sta fLine.ddr_color
	.endif

	.ifdef PutPixel
	mva colorHi,y PutPixel.acol+2

	lda #$1d		; ora *,x
	cpy #0
	sne
	lda #$3d		; and *,x

	sta PutPixel.acol
	.endif
};
end;


procedure PutPixel(x,y: smallint); assembler;
//----------------------------------------------------------------------------------------------
// Puts a point at (X,Y) using color Color
//----------------------------------------------------------------------------------------------
asm
{	stx rx

	cpw x MAIN.SYSTEM.ScreenWidth
	bcs stop

	lda y+1
	bne stop

	ldy y
	lda adr.lineHi,y
	beq stop
	sta bp2+1

	mva adr.lineLo,y bp2

	jmp gr8
mode	equ *-2

; ----------------------------

gr15	ldx x
	ldy adr.div4,x

	jmp plot

; ----------------------------

gr8	ldx x

	lsr x+1
	ror x

	ldy x
	lda adr.div4,y

	tay

plot	lda (bp2),y
acol	ora adr.color_bits,x
	sta (bp2),y	

stop	ldx #0
rx	equ *-1
};
end;


function GetPixel(x,y: smallint): byte; assembler;
//----------------------------------------------------------------------------------------------
// Return color of pixel
//----------------------------------------------------------------------------------------------
asm
{
};
end;


procedure Line(x1,y1,x2,y2: smallint);
var dx, dy, fraction: smallint;
    stepx, stepy: shortint;
begin
    if x1<0 then x1:=0;
    if y1<0 then y1:=0;

    if x2<0 then x2:=0;
    if y2<0 then y2:=0;

    dy := y2 - y1;
    dx := x2 - x1;

    if (dy < 0) then begin dy := -dy; stepy := -1 end else stepy := 1;
    if (dx < 0) then begin dx := -dx; stepx := -1 end else stepx := 1; 
    dy := dy shl 1;        // dy is now 2*dy
    dx := dx shl 1;        // dx is now 2*dx

    PutPixel(x1,y1);

    if (dx > dy) then begin

        fraction := dy - (dx shr 1);		// same as 2*dy - dx

        while (x1 <> x2) do begin
           if (fraction >= 0) then begin
               inc(y1, stepy);
               dec(fraction,dx);		// same as fraction -= 2*dx
           end;
           inc(x1, stepx);
           inc(fraction, dy);			// same as fraction -= 2*dy
           
	   PutPixel(x1, y1);
        end;
	
     end else begin

        fraction := dx - (dy shr 1);

        while (y1 <> y2) do begin
           if (fraction >= 0) then begin
               inc(x1, stepx);
               dec(fraction, dy);
           end;
           inc(y1, stepy);
           inc(fraction, dx);
	   
           PutPixel(x1, y1);
        end;
     end;
 
end;


procedure fLine(x0,y0,x1,y1: byte); assembler;
// DRAWTO in Graphics 15
// A quick hack by eru
asm
{
dx	= ztmp
dy	= ztmp+1
tmp	= ztmp+2
todo	= ztmp+3

PIXEL	.MACRO
	ldy adr.div4,x
	lda (bp2),y
	.def :1 = *
	ora adr.color_bits,x
	sta (bp2),y
	.ENDM

PREPARE	.MACRO
	sta todo
	inc todo
	lsr @
	sta tmp
	.ENDM
	
; ==========================================================================
drawto
	txa:pha
	
; check if going right (x1 <= x0)
	ldx x1
	cpx x0
	bcs right
; going left, swap points	
	lda x0
	stx x0
	sta x1
	tax
	ldy y0
	lda y1
	sty y1
	sta y0
right
; compute X delta
	txa
	sec
	sbc x0
	sta dx
; set initial line address
	ldy y0
	lda adr.lineHi,y
	beq stop_
	sta bp2+1
	lda adr.lineLo,y
	sta bp2
; remember x0 in X
	ldx x0
; check if going up or down
	cpy y1
	jcc down

; ----------------------------- UP ----------------------------------
up
; compute Y delta
	lda y0
	sec
	sbc y1
	sta dy
; check if UP UP RIGHT or UP RIGHT RIGHT
	cmp dx
	bcs up_up_right
up_right_right
	lda dx
	PREPARE urr_color
urr_loop
	PIXEL urr_color
	inx			; go 1 pixel right
	lda tmp
	sec
	sbc dy
	sta tmp
	bcs urr_skip
	adc dx
	sta tmp
	lda bp2			; go 1 line up
	sec
	sbc #0
w0	equ *-1
	sta bp2
	bcs *+4
	dec bp2+1
urr_skip
	dec todo
	bne urr_loop

stop_	jmp stop

up_up_right
	lda dy
	PREPARE uur_color
uur_loop
	PIXEL uur_color
	lda bp2	; go 1 line up
	sec
	sbc #0
w1	equ *-1
	sta bp2
	bcs *+4
	dec bp2+1
	lda tmp
	sec
	sbc dx
	bcs uur_skip
	adc dy
	inx			; go 1 pixel right
uur_skip
	sta tmp
	dec todo
	bne uur_loop

	jmp stop

; ----------------------------- DOWN ----------------------------------
down
; compute Y delta
	lda y1
	sec
	sbc y0
	sta dy
; check if DOWN DOWN RIGHT or DOWN RIGHT RIGHT
	cmp dx
	bcs down_down_right
down_right_right
	lda dx
	PREPARE drr_color
drr_loop
	PIXEL drr_color
	inx			; go 1 pixel right
	lda tmp
	sec
	sbc dy
	sta tmp
	bcs drr_skip
	adc dx
	sta tmp
	lda bp2			; go 1 line down
	clc
	adc #0
w2	equ *-1
	sta bp2
	bcc *+4
	inc bp2+1
drr_skip
	dec todo
	bne drr_loop

	jmp stop

down_down_right
	lda dy
	PREPARE ddr_color
ddr_loop
	PIXEL ddr_color
	lda bp2			; go 1 line down
	clc
	adc #0
w3	equ *-1
	sta bp2
	bcc *+4
	inc bp2+1
	lda tmp
	sec
	sbc dx
	bcs ddr_skip
	adc dy
	inx			; go 1 pixel right
ddr_skip
	sta tmp
	dec todo
	bne ddr_loop

stop	pla:tax
};
end;



procedure MoveTo(x, y: smallint); 
//----------------------------------------------------------------------------------------------
// Move cursor to absolute position
//----------------------------------------------------------------------------------------------
begin
	if x<0 then x:=0;
	if y<0 then y:=0;

	tmpX0 := x;
	tmpY0 := y;
end;


procedure LineTo(x, y: smallint); 
//----------------------------------------------------------------------------------------------
// Draw a line starting from current position to a given point
//----------------------------------------------------------------------------------------------
begin
	if x<0 then x:=0;
	if y<0 then y:=0;

	Line(tmpX0, tmpY0, x,y);

	tmpX0 := x;
	tmpY0 := y;
end;


procedure fRectangle(x1, y1, x2, y2: Smallint);
//----------------------------------------------------------------------------------------------
// Draws a rectangle with corners at (X1,Y1) and (X2,Y2), using the current color and style
//----------------------------------------------------------------------------------------------
begin

 fLine(x1,y1,x2,y1);
 fLine(x2,y1,x2,y2);
 fLine(x1,y2,x2,y2);
 fLine(x1,y1,x1,y2);

end;


procedure InitGraph(mode: byte); overload;
//----------------------------------------------------------------------------------------------
// init graphics mode
//----------------------------------------------------------------------------------------------
var window, i: byte;
    width: byte;
begin

window := mode and $10;

mode := mode and $0f;

width:=40;

ScreenHeight := 192;

case mode of

3:
asm
{	mwa #40 MAIN.SYSTEM.ScreenWidth
	mwa #24 MAIN.SYSTEM.ScreenHeight

	mva #10 width

	.ifdef SetColor
	mwa #SetColor.gr15 SetColor.mode
	.endif

	.ifdef PutPixel
	mwa #PutPixel.gr15 PutPixel.mode
	.endif
};

7:
asm
{	mwa #160 MAIN.SYSTEM.ScreenWidth
	mwa #96 MAIN.SYSTEM.ScreenHeight

	.ifdef SetColor
	mwa #SetColor.gr15 SetColor.mode
	.endif
	
	.ifdef PutPixel
	mwa #PutPixel.gr15 PutPixel.mode
	.endif
};

8:
asm
{	mwa #320 MAIN.SYSTEM.ScreenWidth
	mwa #192 MAIN.SYSTEM.ScreenHeight

	.ifdef SetColor
	mwa #SetColor.gr8 SetColor.mode
	.endif

	.ifdef PutPixel
	mwa #PutPixel.gr8 PutPixel.mode
	.endif
};

9..11:
asm
{	mwa #80 MAIN.SYSTEM.ScreenWidth
	mwa #192 MAIN.SYSTEM.ScreenHeight
};

15:
asm
{	mwa #160 MAIN.SYSTEM.ScreenWidth
	mwa #192 MAIN.SYSTEM.ScreenHeight

	.ifdef SetColor
	mwa #SetColor.gr15 SetColor.mode
	.endif

	.ifdef PutPixel
	mwa #PutPixel.gr15 PutPixel.mode
	.endif
};
end;

asm
{	txa:pha

	.ifdef fLine
	lda width
	sta fLine.w0
	sta fLine.w1
	sta fLine.w2
	sta fLine.w3
	.endif


	ldy mode

	ldx #$60	; 6*16
	lda window	; %00010000 with text window
	ora #2		; read

	.nowarn @graphics

; ---	init_tabs

	mwa 88 bp2

	ldy #0
it1	cpy MAIN.SYSTEM.ScreenHeight
	bcc skp

	lda #0
	sta adr.lineLo,y
	sta adr.lineHi,y
	beq cnt

skp	lda bp2+1
	sta adr.lineHi,y
	lda bp2
	sta adr.lineLo,y
	clc
	adc width
	sta bp2
	bcc *+4
	inc bp2+1

cnt	iny
	bne it1

	mva #$55 _col+1

	ldx #3
	stx _and+1
	txa
l0	sta __oras,x
	asl @
	asl @
	dex
	bpl l0
	
	lda mode
	cmp #8
	bne it2

	mva #$ff _col+1

	ldx #7
	stx _and+1
	lda #1
l1	sta __oras,x
	asl @
	dex
	bpl l1

it2	tya
_and	and #3
	tax
	lda __oras,x
	eor #$ff
	sta adr.color_bits+$000,y ; color0
	lda __oras,x
_col	and #$55
	sta adr.color_bits+$100,y ; color1
	lda __oras,x
	and #$aa
	sta adr.color_bits+$200,y ; color2
	lda __oras,x
	and #$ff
	sta adr.color_bits+$300,y ; color3
	tya
	lsr @
	lsr @
	sta adr.div4,y
	iny
	bne it2

	jmp stop

__oras	dta $c0,$30,$0c,$03
	dta 0,0,0,0
	
stop	pla:tax
};

end;


{$i vbxe\initgraph.pas}
{$i vbxe\setvideobank.pas}
{$i vbxe\setcolormap.pas}
{$i vbxe\setcolorcell.pas}

{$i draw\circle.pas}
{$i draw\ellipse.pas}
{$i draw\rectangle.pas}

end.

