unit system;

{

http://www.freepascal.org/docs-html/rtl/system/index-5.html

Abs
ArcTan
binStr
Cos
DPeek
DPoke
Eof
Exp
FilePos
Fillchar
Frac
hexStr
Ln
lowerCase
Move
octStr
Peek
Poke
Random
Round
Seek
SetLength
Sin
Space
Str
StringOfChar
Sqr
Sqrt
Trunc
upCase
Val

}

interface

type
	TString = string[32];

var	IOResult: byte;
	RndSeed: smallint;

	ScreenWidth: word  = 40;
	ScreenHeight: word = 24;

const
	M_PI_2 = 6.283285;	// pi * 2
	D_PI_2 = 1.570796;	// pi / 2

	GTIA = 0;
	VBXE = $80;

	iDLI = 0;
	iVBL = 1;

implementation


function Peek(a: word): byte; register; assembler;
asm
{	ldy #0
	mva (edx),y Result
};
end;


function DPeek(a: word): word; register; assembler;
asm
{	ldy #0
	mva (edx),y Result
	iny
	mva (edx),y Result+1
};
end;


procedure Randomize; assembler;
asm
{	mva $d20a	RndSeed
	mva rtclok+2	RndSeed+1
;	mva rtclok+1	RndSeed+2
;	mva rtclok	RndSeed+3
};
end;


function Random(range: smallint): smallint;
begin

 if range = 0 then Result := 0 else
{  if range = 256 then Result := Peek($d20a) else} begin
  
	RndSeed := $4595 * RndSeed;
	Result := RndSeed mod range;

asm
{	lda range+1
	bpl plus

	lda Result+1
	bmi ok
	bpl sign
	
plus	lda Result+1
	bpl ok

sign	lda #0
	sub Result
	sta Result

	lda #0
	sbc Result+1
	sta Result+1
ok
}; 

  end;

end;


function IntToReal(x: integer): Real; register; assembler;
//----------------------------------------------------------------------------------------------
// Convert an integer value to a real value
//----------------------------------------------------------------------------------------------
asm
{	mva edx+2 result+3
	mva edx+1 result+2
	mva edx result+1
	mva #0 result
};
end;


function Abs(x: Real): Real; register; assembler; overload;
//----------------------------------------------------------------------------------------------
// Abs returns the absolute value of a variable.
// The result of the function has the same type as its argument, which can be any numerical type.
//----------------------------------------------------------------------------------------------
asm
{	lda edx+3
	spl
	jsr negEDX

	mva edx Result
	mva edx+1 Result+1
	mva edx+2 Result+2
	mva edx+3 Result+3
};
end;


function Abs(x: Integer): Integer; register; assembler; overload;
//----------------------------------------------------------------------------------------------
// Abs returns the absolute value of a variable.
// The result of the function has the same type as its argument, which can be any numerical type.
//----------------------------------------------------------------------------------------------
asm
{	lda edx+3
	spl
	jsr negEDX

	mva edx Result
	mva edx+1 Result+1
	mva edx+2 Result+2
	mva edx+3 Result+3
};
end;


function Sqr(x: Real): Real; overload;
//----------------------------------------------------------------------------------------------
// Sqr returns the square of its argument X
//----------------------------------------------------------------------------------------------
begin

 Result := x*x;

end;


function Sqr(x: integer): integer; overload;
//----------------------------------------------------------------------------------------------
// Sqr returns the square of its argument X
//----------------------------------------------------------------------------------------------
begin

 Result := x*x;

end;


function Sqrt(x: Real): Real; overload;
//----------------------------------------------------------------------------------------------
// Sqrt returns the square root of its argument X, which must be positive
//----------------------------------------------------------------------------------------------
var Divisor: Real;
begin
{ Hero's algorithm }

if x < 0.0 then exit;

Result  := x; 
Divisor := 1.0;

while Abs(Result - Divisor) > 0.01 do
  begin
  Divisor := (Result + Divisor) / 2.0;
  Result := x / Divisor;
  end;
end;


function Sqrt(x: Integer): Integer; overload;
//----------------------------------------------------------------------------------------------
// Sqrt returns the square root of its argument X, which must be positive.
//----------------------------------------------------------------------------------------------
var
  Divisor: Integer;
begin
{ Hero's algorithm }

if x < 0 then exit;

Result  := x; 
Divisor := 1;

while Abs(Result - Divisor) > 1 do
  begin
  Divisor := (Result + Divisor) shr 1;
  Result := x div Divisor;
  end;
end;


function ArcTan(value: real): real;
//----------------------------------------------------------------------------------------------
// Arctan returns the Arctangent of X, which can be any Real type. 
// The resulting angle is in radial units.
//----------------------------------------------------------------------------------------------
var x, y: real;
    sign: Boolean;
begin
  sign:=false;
  x:=value;
  y:=0.0;
 
  if (value=0.0) then begin
    Result:=0.0;
    exit;
  end else
   if (x < 0.0) then begin
    sign:=true;
    x:=-x;
   end;
   
  x:=(x-1.0)/(x+1.0);   
  y:=x*x;
  x := ((((((((.0028662257*y - .0161657367)*y + .0429096138)*y - 
             .0752896400)*y + .1065626393)*y - .1420889944)*y + 
             .1999355085)*y - .3333314528)*y + 1.0)*x;
  x:= .785398163397 + x;
  
  if sign then 
   Result := -x
  else
   Result := x;
 
end;


function Exp(x: Real): Real;
//----------------------------------------------------------------------------------------------
// Exp returns the exponent of X, i.e. the number e to the power X
//----------------------------------------------------------------------------------------------
var
  r: Real;
  k: byte;
begin
Result := 0.0;
r := 1.0;

for k := 1 to 8 do
  begin
  Result := Result + r;
  r := r * x / IntToReal(k);
  end;

end;


function Ln(x: Real): Real;
//----------------------------------------------------------------------------------------------
// Ln returns the natural logarithm of the Real parameter X. X must be positive
//----------------------------------------------------------------------------------------------
var
  r, a: Real;
  k: byte;
begin
Result := 0.0;
a := (x - 1.0) / (x + 1.0);
r := a;

for k := 1 to 8 do
  begin
  Result := Result + r / IntToReal(k shl 1 - 1);
  r := r * a * a;
  end;

Result := Result + Result;
end;


function FilePos(var f: file): cardinal; assembler;
asm
{	txa:pha

	mwa f bp2

	ldy #s@file.chanel
	lda (bp2),y
	tax
	lda #38
	sta iccmd,x
	jsr ciov

	sty IOResult

	mva icax3,x result
	mva icax4,x result+1
	mva icax5,x result+2
	mva #$00 result+3

	pla:tax
};
end;


procedure Seek(var f: file; a: cardinal); assembler;
asm
{	txa:pha

	mwa f bp2

	ldy #s@file.chanel
	lda (bp2),y
	tax
	lda #37
	sta iccmd,x

	mva a icax3,x
	mva a+1 icax4,x
	mva a+2 icax5,x

	jsr ciov

	sty IOResult

	pla:tax
};
end;


function Eof(var f: file): Boolean;
var i: cardinal;
    tmp: byte;
begin
	i:=FilePos(f);
asm
{	mwa f bp2

	ldy #s@file.record
	lda (bp2),y
	pha
	lda #1
	sta (bp2),y
	iny
	lda (bp2),y
	pha
	lda #0
	sta (bp2),y

};
	blockread(f, tmp, 1);

	Seek(f, i);
asm
{	ldy #s@file.record
	iny
	pla
	sta (bp2),y
	dey
	pla
	sta (bp2),y

	ldy #s@file.eof
	lda (bp2),y
	sta result
};
end;


function lowerCase(a: char): char;
//----------------------------------------------------------------------------------------------
// Converts a character to lowercase
//----------------------------------------------------------------------------------------------
begin

 case a of
  'A'..'Z': Result := chr(byte(a) + 32)
 else
  Result := a
 end;

end;


function upCase(a: char): char;
//----------------------------------------------------------------------------------------------
// Converts a character to uppercase
//----------------------------------------------------------------------------------------------
begin

 case a of
  'a'..'z': Result := chr(byte(a) - 32)
 else
  Result := a
 end;

end;


procedure Val(const s: TString; var v: integer; var code: byte): assembler;
//----------------------------------------------------------------------------------------------
// Calculate numerical value of a string
//----------------------------------------------------------------------------------------------
asm
{	@StrToInt #adr.s

	tya
	pha

	mwa code bp2
	ldy #0

	pla
	sta (bp2),y

	mwa v bp2

	mva edx (bp2),y+
	mva edx+1 (bp2),y+
	mva edx+2 (bp2),y+
	mva edx+3 (bp2),y
};
end;


function FloatToStr(a: real): ^char; assembler;
//----------------------------------------------------------------------------------------------
// Convert a float value to a string
//----------------------------------------------------------------------------------------------
asm
{	txa:pha

	inx

	@ValueToStr #@printREAL

	mwa #@buf Result

	pla:tax
};
end;


procedure Str(a: integer; var s: TString); assembler;
//----------------------------------------------------------------------------------------------
// Convert a numerical value to a string
//----------------------------------------------------------------------------------------------
asm
{	txa:pha

	inx

	@ValueToStr #@printINT

	@move #@buf s #16	; !!! koniecznie przez wskaznik

	pla:tax
};
end;


procedure Poke(a: word; value: byte); register; assembler;
asm
{	ldy #0
	mva value (edx),y
};
end;


procedure DPoke(a: word; value: word); register; assembler;
asm
{	ldy #0
	mva value (edx),y
	iny
	mva value+1 (edx),y
};
end;


procedure FillChar(a: pointer; count: word; value: char); assembler; register; overload;
//----------------------------------------------------------------------------------------------
// Fills the memory starting at A with Count Characters with value equal to Value
//----------------------------------------------------------------------------------------------
asm
{	txa:pha

	jsr @fill

stop	pla:tax
};
end;


procedure FillChar(a: pointer; count: word; value: byte); assembler; overload;
//----------------------------------------------------------------------------------------------
// Fills the memory starting at A with Count Bytes with value equal to Value
//----------------------------------------------------------------------------------------------
asm
{	txa:pha

	@fill a count value

stop	pla:tax
};
end;


procedure FillChar(a: pointer; count: word; value: Boolean); assembler; register; overload;
//----------------------------------------------------------------------------------------------
// Fills the memory starting at A with Count Boolean with value equal to Value
//----------------------------------------------------------------------------------------------
asm
{	txa:pha

	jsr @fill

stop	pla:tax
};
end;


procedure Move(source, dest: pointer; count: word); assembler; register;
//----------------------------------------------------------------------------------------------
// Moves Count bytes from Source to Dest
//----------------------------------------------------------------------------------------------
asm
{	jsr @move
};
end;


function Sin(x: Real): Real;
var
  r, x2: Real;
  k, a: byte;
begin
while x > M_PI_2 do x := x - M_PI_2;
while x < 0.0    do x := x + M_PI_2;

Result := 0.0;
r := x;
x2 := x * x;

for k := 1 to 10 do
  begin
  Result := Result + r;
  a := k shl 1;
  r := (-r) * x2 / IntToReal((a + 1) * a);
  end;

end;


function Cos(x: Real): Real;
begin

 Result := Sin(x+ D_PI_2);

end;


function Frac(x: real): real; register; assembler;
//----------------------------------------------------------------------------------------------
// Return fractional part of floating point value
//----------------------------------------------------------------------------------------------
asm
{	@frac

	mva edx result
	mva edx+1 result+1
	mva edx+2 result+2
	mva edx+3 result+3
};
end;


function Trunc(x: real): integer; register; assembler;
//----------------------------------------------------------------------------------------------
// Truncate a floating point value
//----------------------------------------------------------------------------------------------
asm
{	@trunc

	mva edx result
	mva edx+1 result+1
	mva edx+2 result+2
	mva edx+3 result+3
};
end;


function Round(x: real): integer; register; assembler;
//----------------------------------------------------------------------------------------------
// Round floating point value to nearest integer number
// Result :=  cardinal(x) shr FracBits;
//----------------------------------------------------------------------------------------------
asm
{	@round

	mva edx result
	mva edx+1 result+1
	mva edx+2 result+2
	mva edx+3 result+3
};
end;


function Space(b: Byte): ^char; assembler;
//----------------------------------------------------------------------------------------------
// Return a string of spaces
//----------------------------------------------------------------------------------------------
asm
{	ldy #0
	lda #' '
	sta:rne @buf,y+

	mva b @buf

	mwa #@buf Result
};
end;


function StringOfChar(c: Char; l: byte): ^char; assembler;
asm
{	ldy #0
	lda c
	sta:rne @buf,y+

	mva l @buf

	mwa #@buf Result
};
end;


procedure SetLength(var S: TString; Len: byte);
begin

 s[0]:=chr(Len);

end;


function binStr(Value: cardinal; Digits: byte): TString; assembler;
asm
{	txa:pha

	ldy Digits
	cpy #32
	scc
	ldy #32

	sty @buf

_tob1	lda #0
	lsr Value+3
	ror Value+2
	ror Value+1
	ror Value
	adc #'0'
	sta @buf,y
	dey
	bne _tob1

	@move #@buf Result #33

	pla:tax
};
end;


function octStr(Value: cardinal; Digits: byte): TString; assembler;
asm
{	txa:pha

	ldy Digits
	cpy #32
	scc
	ldy #32

	sty @buf

_toct1	ldx #3
	lda #0
_toct2	lsr Value+3
	ror Value+2
	ror Value+1
	ror Value
	ror @
	dex
	bne _toct2

	:5 lsr @

	ora #'0'
	sta @buf,y

	dey
	bne _toct1

	@move #@buf Result #33

	pla:tax
};
end;


function hexStr(Value: cardinal; Digits: byte): TString; register; assembler;
asm
{	txa:pha

	jsr @hexStr

	@move #@buf Result #33

	pla:tax
};
end;

end.

