unit sysutils;

{

http://www.freepascal.org/docs-html/rtl/sysutils/index-5.html

DeleteFile
FileExists
FindFirst
FindNext
FindClose
IntToHex
IntToStr
RenameFile
StrToFloat
StrToInt

}

interface

type
TSearchRec    = record
		Attr: Byte;
		Name: TString;
		FindHandle: Pointer;
		end;

const
	faReadOnly	= $01;
	faHidden	= $02;
	faSysFile	= $04;
	faVolumeID	= $08;
	faDirectory	= $10;
	faArchive	= $20;
	faAnyFile	= $3f;


implementation


function FindFirst (const FileMask: TString; Attributes: Byte; var SearchResult: TSearchRec): byte;
var f: file;
begin
	assign(f, FileMask);
asm
{	txa:pha

	@openfile f #6

	mwa SearchResult bp2
	ldy #SearchResult.FindHandle-DATAORIGIN

	lda f
	sta (bp2),y
	iny
	lda f+1
	sta (bp2),y

	mwa f bp2

	ldy #s@file.record
	mva <1 (bp2),y
	iny
	mva >1 (bp2),y

	ldy #s@file.nrecord
	mva <64 (bp2),y
	iny
	mva >64 (bp2),y

	ldy #s@file.buffer
	mva <@buf (bp2),y
	iny
	mva >@buf (bp2),y

	@ReadDirFileName f
	sta Result

	adw SearchResult #SearchResult.Name-DATAORIGIN bp2

	jsr @DirFileName

	pla:tax
};
end;


function FindNext(var f: TSearchRec): byte; assembler;
asm
{	txa:pha

	mwa f bp2
	ldy #f.FindHandle-DATAORIGIN
	mva (bp2),y edx
	iny
	mva (bp2),y edx+1

	@ReadDirFileName edx
	sta Result

	adw f #f.Name-DATAORIGIN bp2

	jsr @DirFileName

	pla:tax	
};
end;


procedure FindClose(var f: TSearchRec); assembler;
asm
{	txa:pha

	mwa f bp2
	ldy #f.FindHandle-DATAORIGIN
	mva (bp2),y edx
	iny
	mva (bp2),y edx+1

	@closefile edx

	pla:tax
};
end;


function RenameFile(var OldName,NewName: TString): Boolean; assembler;
asm
{	txa:pha

	mva #0 @buf

	@addString OldName
	ldy @buf
	lda #','
	sta @buf+1,y
	inc @buf
	@addString NewName

	jsr @openfile.lookup
	bmi stop

	lda #32
	sta iccmd,x
	lda <@buf+1
	sta icbufa,x
	lda >@buf+1
	sta icbufa+1,x
	lda #$00
	sta icax1,x
	sta icax2,x
	jsr ciov

stop	sty MAIN.SYSTEM.IOResult

	bpl ok

	lda #false
	seq

ok	lda #true
	sta Result

	pla:tax
};
end;


function DeleteFile(var FileName: TString): Boolean; assembler;
asm
{	txa:pha

	jsr @openfile.lookup
	bmi stop

	lda #33
	sta iccmd,x
	lda FileName
	add #1
	sta icbufa,x
	lda FileName+1
	adc #0
	sta icbufa+1,x
	lda #$00
	sta icax1,x
	sta icax2,x
	jsr ciov

stop	sty MAIN.SYSTEM.IOResult

	bpl ok

	lda #false
	seq

ok	lda #true
	sta Result

	pla:tax
};
end;


function FileExists(var name: TString): Boolean;
var f: file;
begin

  {$I-}
  Assign (f, name);
  Reset (f);
  {$I+}
  Result:=(IoResult<128) and (length(name)>0);
  Close (f);

end;


function IntToStr(a: integer): ^char; assembler; 
//----------------------------------------------------------------------------------------------
// Convert an integer value to a decimal string
//----------------------------------------------------------------------------------------------
asm
{	txa:pha

	inx

	@ValueToStr #@printINT

	mwa #@buf Result

	pla:tax
};
end;


function StrToInt(const s: char): byte; assembler; overload;
//----------------------------------------------------------------------------------------------
// Convert a string to an integer value
//----------------------------------------------------------------------------------------------
asm
{	mva s @buf+1
	mva #1 @buf

	@StrToInt #@buf

	mva edx Result
	mva edx+1 Result+1
	mva edx+2 Result+2
	mva edx+3 Result+3
};
end;


function StrToInt(const s: TString): integer; assembler; overload;
//----------------------------------------------------------------------------------------------
// Convert a string to an integer value
//----------------------------------------------------------------------------------------------
asm
{	@StrToInt #adr.s

	mva edx Result
	mva edx+1 Result+1
	mva edx+2 Result+2
	mva edx+3 Result+3
};
end;


function IntToHex(Value: cardinal; Digits: byte): TString; register; assembler;
//----------------------------------------------------------------------------------------------
// Convert an integer value to a hexadecimal string
//----------------------------------------------------------------------------------------------
asm
{	txa:pha

	jsr @hexStr

	@move #@buf Result #33

	pla:tax
};
end;


function StrToFloat(var s: TString): real;
//----------------------------------------------------------------------------------------------
// Convert a string to a floating-point value
//----------------------------------------------------------------------------------------------
var n, dotpos, len: byte;
begin

 result:=0.0;

 len:=length(s) + 1;

 if len > 1 then begin

	dotpos:=0;

	if (s[1] = '-') or (s[1] = '+') then	//Added line to check sign.If the number is signed,
		n:=2				//set n to position 2.
	else					//(number is not signed)
		n:=1;				//set n to position 1.

// If the number was signed,then we set n to 2,
// so that we start with s[2],and at the end
// if the number was negative we will multiply by -1.

	while n<len do begin			//n is already set to the position of the fisrt number.

	if (s[n] = '.') then
		dotpos := len - n - 1
        else
		result := result * 10.0 +  IntToReal(ord(s[n])-ord('0'));

	inc(n);
	end;

	while dotpos <> 0 do begin
		result := result / 10.0;
		dec(dotpos);
	end;

	if (s[1]='-') then			//If s[] is "negative",
	result :=  real(not( cardinal(result)));//multiply the number by -1,making it also negative.

 end;

end;

end.

